#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXSize.h"
#include "FXId.h"
#include "FXALDevice.h"
#include "FXALContext.h"

// Object implementation
FXIMPLEMENT(FXALContext,FXId,NULL,0)

FXALContext::FXALContext(FXApp * a,FXALDevice * d,FXint f,FXint r, FXint s) : FXId(a)
{
	FXASSERT(d);

	frs[0] = f;
	frs[1] = r;
	frs[2] = s;
	
	pDevice = d;

	ctx = NULL;
}

FXALContext::~FXALContext()
{
#ifdef HAVE_OPENAL
	destroy();
#endif
}

void FXALContext::create()
{
#ifdef HAVE_OPENAL

	FXASSERT(((FXALDevice*)pDevice)->dev);

	ALCdevice * dev = (ALCdevice*) pDevice->dev;
	
	//detach possible context
	detach();
	
	//if there are attributes, try them...
	if (frs[0] != -1 && 
		frs[1] != -1 && 
		frs[2] != -1)
	{
		ctx = alcCreateContext(dev,frs);
		if (!ctx) printf("\nFXALContext::create : %s",alcGetString(dev,alcGetError(dev)));
	}
	
	
	//no attributes or didn't work, make default context
	if (!ctx)
	{
		ctx = alcCreateContext(dev,NULL);
		if (!ctx) printf("\nFXALContext::create : %s",alcGetString(dev,alcGetError( dev)));
	}
#endif
}

void FXALContext::destroy()
{
#ifdef HAVE_OPENAL
	if (ctx)
	{
		makeNonCurrent(); 
		alcDestroyContext((ALCcontext*)ctx);
	}
#endif
}

bool FXALContext::makeCurrent()
{
#ifdef HAVE_OPENAL
	if (!ctx) return false;
	if (!alcMakeContextCurrent((ALCcontext*)ctx))
	{
		
	//	printf("\nFXALContext::MakeCurrent() : %s",alcGetString(((FXALDevice*)pDevice)->dev,alcGetError( ((FXALDevice*)pDevice)->dev )));
		return false;
	}
	return true;
#endif
	return false;
}

bool FXALContext::makeNonCurrent()
{
#ifdef HAVE_OPENAL
	alGetError();
	if (!alcMakeContextCurrent(NULL))
	{
	//	printf("\nFXALContext::MakeNonCurrent() : %s",alGetString( alGetError()));
		return false;
	}
	return true;
#endif
	return false;
}

void FXALContext::suspend()
{
#ifdef HAVE_OPENAL
	if (ctx) alcSuspendContext((ALCcontext*)ctx);
#endif
}


void FXALContext::process()
{
#ifdef HAVE_OPENAL
	if (ctx) alcProcessContext((ALCcontext*)ctx);
#endif
}
